import * as i0 from '@angular/core';
import { EventEmitter, Directive, Input, Output } from '@angular/core';
import { Subject, from, fromEvent } from 'rxjs';
import { mergeMap, map, filter, debounceTime, distinctUntilChanged, takeUntil } from 'rxjs/operators';

class NgxScanDetectDirective {
    constructor(elementRef) {
        this.elementRef = elementRef;
        /**
         * Working Mode
         */
        this.workMode = 'manual';
        /**
         * Input delay
         */
        this.debounce = 0;
        /**
         * After how many characters start search
         */
        this.skipStart = 0;
        /**
         * Data cleansing event
         */
        this.delete = new EventEmitter();
        /**
         * Event after data entry
         */
        this.detected = new EventEmitter();
        /**
         * Use for unsubscribe
         */
        this.destroy$ = new Subject();
    }
    ngAfterViewInit() {
        /**
         * Often the code scanner is connected to the computer.
         * It emulates a press key, so we use keyboard events to press and release keys.
         */
        const events = ['keydown', 'keyup'];
        /**
         * Empty object for delay logic
         */
        const pressed = {};
        /**
         * Look at the
         * {@Link http://reactivex.io/documentation/operators/from.html}
         */
        from(events)
            .pipe(
        /**
         * Look at the
         * {@Link https://rxjs-dev.firebaseapp.com/api/operators/mergeMap}
         */
        mergeMap((event) => fromEvent(this.elementRef.nativeElement, event)), 
        /**
         * Prepare input data
         */
        map((event) => {
            switch (event.type) {
                case 'keydown':
                    pressed[event.key] = event.timeStamp;
                    break;
                case 'keyup':
                    /**
                     * In the delay set the difference between keydown and keyup events.
                     */
                    Object.assign(event, {
                        duration: (event.timeStamp - +pressed[event.key]) / 1000,
                    });
                    break;
            }
            /**
             * @return {event: KeyboardEvent}
             */
            return event;
        }), filter((event) => {
            if (event.key === 'Backspace' || event.code === 'Backspace') {
                /**
                 * Used to clear data.
                 */
                this.delete.emit({
                    event,
                    value: event.target.value,
                    type: 'delete',
                });
            }
            /**
             * Return data after typed in two characters.
             */
            return (event.target instanceof HTMLInputElement &&
                event.target.value.length > this.skipStart);
        }), 
        /**
         * Data entry delay is used to limit requests.
         */
        debounceTime(this.debounce), 
        /**
         * Look at the
         * {@Link http://reactivex.io/documentation/operators/distinct.html}
         */
        distinctUntilChanged())
            /**
             * Use for unsubscribe
             */
            .pipe(takeUntil(this.destroy$))
            /**
             * Subscribe to the input data and determine the delay time for our purposes.
             */
            .subscribe((event) => {
            this.detected.emit({
                event,
                value: event.target.value,
                time: event.duration,
                type: event.duration > 0.02 ? 'keyboard' : 'scanner',
            });
            if (this.workMode === 'multiple') {
                /**
                 * Clear input value on each event in 'multiple' mode
                 */
                event.target.value = '';
            }
        });
    }
    /**
     * Use for unsubscribe
     */
    ngOnDestroy() {
        this.destroy$.next(true);
        this.destroy$.complete();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgxScanDetectDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgxScanDetectDirective, isStandalone: true, selector: "input[ngxScanDetect]", inputs: { workMode: "workMode", debounce: "debounce", skipStart: "skipStart" }, outputs: { delete: "delete", detected: "detected" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgxScanDetectDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[ngxScanDetect]',
                    standalone: true,
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { workMode: [{
                type: Input
            }], debounce: [{
                type: Input
            }], skipStart: [{
                type: Input
            }], delete: [{
                type: Output
            }], detected: [{
                type: Output
            }] } });

/*
 * Public API Surface of ngx-scan-detect
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxScanDetectDirective };
//# sourceMappingURL=ngx-scan-detect.mjs.map
